@@ -1,17 +1,19 @@
 import logging as _logging
 
+from pyqtgraph import mkQApp
+
 from qtextras._funcparse import (
+    FROM_PREV_IO,
     ParameterlessInteractor,
     QtExtrasInteractor,
     bindInteractorOptions,
-    FROM_PREV_IO,
 )
-from qtextras.constants import PrjEnums
+from qtextras.constants import LibraryNamespace
+from qtextras.fns import *
 from qtextras.misc import *
 from qtextras.params import *
 from qtextras.widgets import *
-from qtextras.fns import *
 
-_logging.addLevelName(PrjEnums.LOG_LVL_ATTN, "ATTN")
+_logging.addLevelName(LibraryNamespace.LOG_LEVEL_ATTENTION, "ATTENTION")
 
-__version__ = "0.6.3"
+__version__ = "0.6.7"
