@@ -10,9 +10,8 @@ from contextlib import contextmanager
 from pathlib import Path
 from warnings import warn
 
-import pandas as pd
 import pyqtgraph as pg
-from pyqtgraph.parametertree import Parameter, ParameterTree
+from pyqtgraph.parametertree import InteractiveFunction, Parameter, ParameterTree
 from pyqtgraph.Qt import QT_LIB, QtCore, QtGui, QtWidgets
 
 # -----
@@ -21,6 +20,7 @@ from pyqtgraph.Qt import QT_LIB, QtCore, QtGui, QtWidgets
 # Needed for external modules to catch a YamlError
 from ruamel.yaml import YAML, YAMLError  # noqa
 
+from qtextras.shims import pd, requires_pandas
 from qtextras.typeoverloads import FilePath
 
 # -----
@@ -189,6 +189,7 @@ def dynamicDocstring(embed=False, **kwargs):
     return wrapper
 
 
+@requires_pandas
 def seriesAsFrame(ser: pd.Series):
     return ser.to_frame().T
 
@@ -209,7 +210,7 @@ def createAndAddMenuAct(
         return act
 
 
-def pascalCaseToTitle(name: str, addSpaces=True) -> str:
+def pascalCaseToTitle(name: str, addSpaces=True, onlyOnAlphanumeric=True) -> str:
     """
     Helper utility to turn a PascalCase name to a 'Title Case' title
 
@@ -219,6 +220,10 @@ def pascalCaseToTitle(name: str, addSpaces=True) -> str:
         camel-cased name
     addSpaces
         Whether to add spaces in the final result
+    onlyOnAlphanumeric
+        Whether to only perform processing when the input consists totally of alphanumeric
+        characters (plus underscores). When ``True``, if the string contains characters
+        such as "(", ".", etc., the original innput will be returned unformatted.
 
     Returns
     -------
@@ -230,7 +235,9 @@ def pascalCaseToTitle(name: str, addSpaces=True) -> str:
     # changeROIImage -> change ROI Image
     # HTTPServer -> HTTP Server
     # exportData -> export Data
-    name = re.sub(r"([A-Z]?[a-z0-9]+)", r" \1 ", name)
+    if onlyOnAlphanumeric and not name.replace("_", "").isalnum():
+        return name
+    name = re.sub(r"([A-Z]?[^A-Z_]+)", r" \1 ", name)
     name = name.replace("_", " ")
     # title() would turn HTTPS -> Https which we don't want
     parts = [p[0].upper() + p[1:] for p in name.split()]
@@ -289,6 +296,19 @@ class _IGNORE_VALUE:
     """
 
 
+VALUE_GROUP_TYPES = ["group", "_actiongroup", "chainedactiongroup"]
+FLATTEN_GROUP_TYPES = ["group"]
+
+
+def _isGroup(param, groupTypes):
+    for typ in groupTypes:
+        if typ == param.type():
+            return True
+        elif callable(typ) and typ(param):
+            return True
+    return False
+
+
 def valueIsNotDefault(parameter):
     """
     Convenience function to pass to ``parameterValues`` as a ``valueFilter`` without
@@ -301,7 +321,7 @@ def parameterValues(
     parameter: Parameter,
     valueFilter: t.Callable[[Parameter], bool | t.Any] = None,
     value="value",
-    groupTypes: t.Sequence[str] = ("group",),
+    groupTypes: t.Sequence[str, t.Callable] = None,
 ) -> t.Any | dict[str, t.Any]:
     """
     Returns just parameter values in a human-readable fashion. A group parameter's
@@ -320,9 +340,13 @@ def parameterValues(
         the enabled state of a parameter
     groupTypes
         The types of parameters that should be considered groups when deciding whether to
-        recurse into them
+        recurse into them. If a callable is passed, it will be called with the parameter
+        and should return ``True`` if the parameter should be considered a group.
     """
-    returnValueOnly = parameter.type() not in groupTypes
+
+    if groupTypes is None:
+        groupTypes = VALUE_GROUP_TYPES
+    returnValueOnly = not _isGroup(parameter, groupTypes)
     # Some values are not well represented as text, so call 'saveState' to ensure
     # they work
     if returnValueOnly and (not valueFilter or valueFilter(parameter)):
@@ -332,9 +356,9 @@ def parameterValues(
     # Else is a group parameter
     outDict = {}
     for child in parameter:
-        isGroup = child.type() in groupTypes
+        childIsGroup = _isGroup(child, groupTypes)
         chState = parameterValues(child, valueFilter, value, groupTypes)
-        if (chState is _IGNORE_VALUE) or (isGroup and not chState):
+        if (chState is _IGNORE_VALUE) or (childIsGroup and not chState):
             # Don't include empty groups or ignored child values
             continue
         outDict[child.name()] = chState
@@ -409,11 +433,15 @@ def applyParameterOpts(parameter: Parameter, opts: dict):
             applyParameterOpts(parameter.child(chName), chDict)
 
 
-def flattenedParameters(parameter: Parameter, groupTypes: t.Sequence[str] = ("group",)):
+def flattenedParameters(
+    parameter: Parameter, groupTypes: t.Sequence[str, t.Callable] = None
+):
+    if groupTypes is None:
+        groupTypes = FLATTEN_GROUP_TYPES
     addList = []
-    if parameter.type() in groupTypes:
+    if _isGroup(parameter, groupTypes):
         for child in parameter.children():  # type: Parameter
-            addList.extend(flattenedParameters(child))
+            addList.extend(flattenedParameters(child, groupTypes))
     else:
         addList.append(parameter)
     return addList
@@ -469,6 +497,40 @@ def setParametersExpanded(tree: ParameterTree, expandedVal=True):
     tree.resizeColumnToContents(0)
 
 
+def interactAndHandleExistingParameters(
+    interactor, function: InteractiveFunction, allowSetValue=False, **kwargs
+):
+    existingParams = function.parameters.copy()
+    # Set to nothing before interacting, so they can be added back without
+    # being disconnected
+    function.parameters = {}
+    if alreadyRegistered := set(existingParams).intersection(kwargs):
+        if allowSetValue:
+            wasDisconncted = function.disconnect()
+            try:
+                for kwarg in alreadyRegistered:
+                    existingParams[kwarg].setValue(kwargs[kwarg])
+            finally:
+                if not wasDisconncted:
+                    function.reconnect()
+            for param in alreadyRegistered:
+                existingParams[param].setValue(kwargs[param])
+        else:
+            raise ValueError(
+                f"Cannot pass parameters {alreadyRegistered} that are already "
+                f"registered to {function}"
+            )
+    kwargs["ignores"] = set(kwargs.get("ignores", ())) | set(existingParams)
+    out = interactor(function, **kwargs)
+    function.parameters.update(existingParams)
+    needsChild = isinstance(out, Parameter)
+    for ii, (name, param) in enumerate(existingParams.items()):
+        needsChild and out.insertChild(ii, param)
+        function.extra.pop(name, None)
+        function.parameterCache[name] = param.value()
+    return out
+
+
 def forceRichText(text: str):
     """
     Wraps text in <qt> tags to make Qt treat it as rich text. Since tooltips don't wrap
@@ -499,7 +561,7 @@ def setParamTooltips(tree: ParameterTree, expandNameCol=False):
         #   lbl = QtWidgets.QLabel(item.text(0))
         #   tree.setItemWidget(item, 0, lbl)
         if (
-            hasattr(item, "parameter")
+            hasattr(item, "param")
             and "tip" in item.param.opts
             and len(item.toolTip(0)) == 0
             and tree.itemWidget(item, 0) is None
@@ -645,14 +707,14 @@ def makeCli(
         Passed to AtomicProcess which is used as an intermediate to generate parameter
         documentation
     """
-    from qtextras._funcparse import FROM_PREV_IO, QtExtrasInteractor
+    from qtextras._funcparse import FROM_PREV_IO, DocstringInteractor
 
     def tipConverter(inStr):
         doc = QtGui.QTextDocument()
         doc.setHtml(inStr)
         return doc.toPlainText()
 
-    funcDict = QtExtrasInteractor().functionToParameterDict(func, **procKwargs)
+    funcDict = DocstringInteractor().functionToParameterDict(func, **procKwargs)
     tip = funcDict.get("tip", "")
 
     useCls = argparse.ArgumentParser if not convertArgs else ArgParseConverter
@@ -881,7 +943,7 @@ def overrideAttr(attrHolder, attrName: str, newVal):
     oldVal = getattr(attrHolder, attrName)
     try:
         setattr(attrHolder, attrName, newVal)
-        yield
+        yield oldVal
     finally:
         setattr(attrHolder, attrName, oldVal)
 
