@@ -11,8 +11,8 @@ from pyqtgraph.parametertree import InteractiveFunction, Parameter
 from pyqtgraph.parametertree.interactive import Interactor
 from pyqtgraph.Qt import QtCore, QtGui, QtWidgets
 
-from qtextras import fns
-from qtextras._funcparse import QtExtrasInteractor, RunOptions
+from qtextras import VALUE_GROUP_TYPES, fns
+from qtextras._funcparse import DocstringInteractor, RunOptions
 from qtextras.params.optionsdict import OptionsDict
 from qtextras.typeoverloads import FilePath
 from qtextras.widgets.easywidget import EasyWidget
@@ -133,29 +133,36 @@ class ParameterStateSignals(QtCore.QObject):
 class FileStateManager:
     DEFAULT_STATE_NAME = "Default"
 
+    allowBrowseForLoad = True
+
     def __init__(
         self,
         directory: FilePath | None = None,
         suffix: str = ".param",
     ):
-        if directory:
-            directory = Path(directory)
-
         self.signals = ParameterStateSignals()
-        self.directory = directory
+        self.directory: Path | None = None
         self.suffix = suffix
 
-        self.watcher: QtCore.QFileSystemWatcher | None = None
         self.savedStates: list[str] = []
         """Records which states are available since the last change"""
 
-        if self.directory is not None:
-            self.watcher = QtCore.QFileSystemWatcher()
-            self.watcher.addPath(str(self.directory.resolve()))
-            # False positive type hint issues, return values are allowed
-            self.watcher.directoryChanged.connect(self.updateSavedStates)  # noqa
-            self.watcher.fileChanged.connect(self._onWatcherFileChanged)
-            self.updateSavedStates()
+        self.watcher = QtCore.QFileSystemWatcher()
+        # False positive type hint issues, return values are allowed
+        self.watcher.directoryChanged.connect(self.updateSavedStates)  # noqa
+        self.watcher.fileChanged.connect(self._onWatcherFileChanged)
+        self.moveDirectory(directory)
+
+    def moveDirectory(self, newDirectory: FilePath = None):
+        if newDirectory is not None:
+            newDirectory = Path(newDirectory)
+        self.directory = newDirectory
+        if toRemove := self.watcher.directories():
+            self.watcher.removePaths(toRemove)
+        if newDirectory is None:
+            return
+        self.watcher.addPath(str(self.directory.resolve()))
+        self.updateSavedStates()
 
     def updateSavedStates(self):
         """
@@ -233,7 +240,7 @@ class FileStateManager:
             watch = False
         if addDefaults:
             stateDict = self.maybePopulateDefaults(stateDict)
-        if watch and self.watcher:
+        if watch:
             if files := self.watcher.files():
                 self.watcher.removePaths(files)
             self.watcher.addPath(str(self.directory))
@@ -283,6 +290,12 @@ class FileStateManager:
             curAction = parentMenu.addAction(name)
             curAction.triggered.connect(functools.partial(loaderFunc, name))
 
+        if self.allowBrowseForLoad:
+            parentMenu.addSeparator()
+            parentMenu.addAction("Browse...").triggered.connect(
+                functools.partial(self.signals.loadRequested.emit, "")
+            )
+
         if keepUpdated:
             self.signals.created.connect(populateFunc)
             self.signals.deleted.connect(populateFunc)
@@ -299,7 +312,7 @@ class ParameterEditor(QtWidgets.QWidget):
     name: str = None
     """Human readable name (i.e., used when inserting into a menu)"""
 
-    defaultInteractor = QtExtrasInteractor()
+    defaultInteractor = DocstringInteractor()
     defaultParent: _ParentType = None
 
     def __init__(
@@ -453,7 +466,9 @@ class ParameterEditor(QtWidgets.QWidget):
         self, loadDict: dict, candidateParameters: t.Sequence[Parameter] = None
     ):
         if candidateParameters is None:
-            candidateParameters = fns.flattenedParameters(self.rootParameter)
+            candidateParameters = fns.flattenedParameters(
+                self.rootParameter, groupTypes=VALUE_GROUP_TYPES
+            )
         if not candidateParameters:
             return
 
@@ -546,6 +561,16 @@ class ParameterEditor(QtWidgets.QWidget):
     def onLoadRequested(self, stateName):
         # First check for extra keys, will be the case if 'children' is one of the
         # keys. Can't do value-loading in that case, must do state-loading instead
+        if not stateName:
+            # No state name given, get from the user
+            stateName, _ = QtWidgets.QFileDialog.getOpenFileName(
+                self,
+                "Load State",
+                str(self.directory),
+                f"State files (*{self.stateManager.suffix});; All files (*)",
+            )
+            if not stateName:
+                return
         stateDict = self.stateManager.loadState(stateName)
         if not stateDict:
             # File didn't exist or there was a problem during loading
@@ -663,10 +688,10 @@ class ParameterEditor(QtWidgets.QWidget):
             the function's original attribute value is unchanged.
         container
             The container in which to put child parameters from the registered function.
-            This has a similar effect to calling `registerProps` on each of the
+            This has a similar effect to calling `registerParameterList` on each of the
             function arguments with this container. *Note*: while the former will
-            register props according to the `parameter` OptionsDict, this will register
-            to the container using the raw parameter name. A KeyError is raised when
+            register according to the `parameter` OptionsDict, this function will register
+            to the container using the string parameter name. A KeyError is raised when
             a new parameter name clashes with something already in the container.
         **kwargs
             All additional kwargs are passed to InteractiveFunction when wrapping the
@@ -703,7 +728,7 @@ class ParameterEditor(QtWidgets.QWidget):
         if runActionTemplate:
             kwargs["runActionTemplate"] = runActionTemplate
 
-        self._interactAndHandleExistingParameters(
+        fns.interactAndHandleExistingParameters(
             interactor, function, parent=parent, **kwargs
         )
 
@@ -763,25 +788,6 @@ class ParameterEditor(QtWidgets.QWidget):
         shortcuttable.setShortcut(shortcut)
         self.nameShortcutMap[name] = shortcuttable
 
-    @classmethod
-    def _interactAndHandleExistingParameters(cls, interactor, function, **kwargs):
-        existingParams = function.parameters.copy()
-        # Set to nothing before interacting, so they can be added back without
-        # being disconnected
-        function.parameters = {}
-        if alreadyRegistered := set(existingParams).intersection(kwargs):
-            raise ValueError(
-                f"Cannot pass parameters {alreadyRegistered} that are already "
-                f"registered to {function}"
-            )
-        kwargs["ignores"] = set(kwargs.get("ignores", ())) | set(existingParams)
-        interactor(function, **kwargs)
-        function.parameters.update(existingParams)
-        for ii, (name, param) in enumerate(existingParams.items()):
-            kwargs["parent"].insertChild(ii, param)
-            del function.extra[name]
-            function.parameterCache[name] = param.value()
-
     def createWindowDock(
         self,
         window: QtWidgets.QMainWindow,
@@ -946,13 +952,14 @@ class ParameterEditor(QtWidgets.QWidget):
         ]
 
     def _createLoadButton(self):
-        self.loadButton = QtWidgets.QToolButton()
+        self.loadButton = QtWidgets.QToolButton(self)
         self.loadButton.setText("Load...")
         pol = QtWidgets.QSizePolicy.Policy
         self.loadButton.setSizePolicy(pol.MinimumExpanding, pol.Fixed)
-        self.loadButton.setMenu(QtWidgets.QMenu(self.loadButton))
+        menu = QtWidgets.QMenu(self.loadButton)
+        self.loadButton.setMenu(menu)
         self.loadButton.setPopupMode(QtWidgets.QToolButton.InstantPopup)
-        self.stateManager.addSavedStatesToMenu(self.loadButton.menu())
+        self.stateManager.addSavedStatesToMenu(menu)
         return self.loadButton
 
     def __repr__(self):
