@@ -3,7 +3,6 @@ from __future__ import annotations
 import typing as t
 
 import numpy as np
-import pandas as pd
 import pyqtgraph as pg
 from _warnings import warn
 from pyqtgraph import QtCore, QtGui, QtWidgets
@@ -15,6 +14,7 @@ from qtextras import fns
 from qtextras._funcparse import bindInteractorOptions as bind
 from qtextras.misc import CompositionMixin
 from qtextras.params import ParameterEditor, RunOptions
+from qtextras.shims import pd, requires_pandas
 from qtextras.typeoverloads import FilePath
 from qtextras.widgets.easywidget import EasyWidget
 
@@ -34,8 +34,8 @@ class ImageViewer(CompositionMixin, pg.PlotWidget):
         self.menu: QtWidgets.QMenu = vb.menu
         self.oldVbMenu: ViewBoxMenu = vb.menu
         # Disable default menus
-        self.plotItem.ctrlMenu = None
-        self.sceneObj.contextMenu = None
+        # self.plotItem.ctrlMenu = None
+        # self.sceneObj.contextMenu = None
 
         self.setAspectLocked(True)
         vb.invertY()
@@ -137,7 +137,9 @@ class ImageViewer(CompositionMixin, pg.PlotWidget):
             self.pxColorLbl = QtWidgets.QLabel()
             wid.statusBar().addWidget(self.mouseCoordsLbl)
             wid.statusBar().addWidget(self.pxColorLbl)
-            wid.addDockWidget(QtCore.Qt.RightDockWidgetArea, self.toolsEditor)
+            dock = QtWidgets.QDockWidget("Tools", wid)
+            dock.setWidget(self.toolsEditor)
+            wid.addDockWidget(QtCore.Qt.RightDockWidgetArea, dock)
             if showTools:
                 self.toolsEditor.show()
         else:
@@ -178,10 +180,11 @@ class CompositorLegend(pg.LegendItem):
         p.drawRoundedRect(br, 5, 5)
 
 
+@requires_pandas
 class MaskCompositor(ImageViewer):
     _cachedCmapLimits = None
 
-    def __init__(self, img: np.ndarray = None):
+    def __init__(self, img: np.ndarray = None, registerDefaultFunctions=False):
         super().__init__()
         # -----
         # Create properties
@@ -194,16 +197,18 @@ class MaskCompositor(ImageViewer):
             type(self)._cachedCmapLimits = fns.listAllPgColormaps()
 
         self.legend = CompositorLegend(
-            offset=(5, 5), horSpacing=5, brush="ccce", pen="k"
+            offset=(5, 5), horSpacing=5, brush="#ccce", pen="k"
+        )
+        self.recordDf = (
+            pd.DataFrame(columns=["name", "item", "opacity", "isMask"])
+            .set_index("name")
+            .astype(dict(isMask="bool"))
         )
-        self.recordDf = pd.DataFrame(
-            columns=["name", "item", "opacity", "isMask"]
-        ).set_index("name")
         # Initialized in clearOverlays and updateLabelMap. Given here to avoid
         # intellisense warnings
         self.inUseLabelValues = (
             self.scatterSer
-        ) = self.backgroundValues = self.labelToNameMapping = None
+        ) = self.backgroundValues = self.labelNameMap = None
         self.updateLabelMap()
         self.legendFontArgs = {"size": "11pt", "color": "k", "bold": True}
         self.curZ = 2
@@ -212,44 +217,39 @@ class MaskCompositor(ImageViewer):
         # Configure relationships
         # -----
         self.legend.setParentItem(self.plotItem)
+        self.legend.hide()
         self.viewbox: pg.ViewBox = self.getViewBox()
 
+        self.compositorMenu = self.menu.addMenu("Compositor Options")
+        for func in self.toggleLegendVisible, self.toggleImageVisible:
+            self.compositorMenu.addAction(fns.nameFormatter(func.__name__), func)
+
         self.allVisible = True
         # for ax in 'left', 'bottom', 'right', 'top':
         #   self.mainImage.plotItem.hideAxis(ax)
         # self.mainImage.setContentsMargins(0, 0, 0, 0)
+        self.propertiesProc = self.toolsEditor.registerFunction(
+            self.setOverlayProperties,
+            name=fns.nameFormatter("overlayProperties"),
+            runOptions=RunOptions.ON_CHANGED,
+            colormap=dict(limits=self._cachedCmapLimits),
+            fullLabelMapColors=dict(ignore=True),
+        )
 
-        def newFmt(name):
-            if name.startswith("set"):
-                name = name.replace("set", "")
-            name = fns.pascalCaseToTitle(name)
-            return name
+        if registerDefaultFunctions:
+            self.registerDefaultFunctions()
 
-        with fns.nameFormatter.set(newFmt):
-            self.propertiesProc = self.toolsEditor.registerFunction(
-                self.setOverlayProperties,
-                runOptions=RunOptions.ON_CHANGED,
-                colormap=dict(limits=self._cachedCmapLimits),
-                fullLabelMapColors=dict(ignore=True),
-            )
-            self.toolsEditor.registerFunction(self.save)
-            self.toolsEditor.registerFunction(
-                lambda: self.imageItem.setVisible(not self.imageItem.isVisible()),
-                name="Toggle Image Visible",
-            )
-            self.toolsEditor.registerFunction(
-                lambda: self.legend.setVisible(not self.legend.isVisible()),
-                name="Toggle Legend Visible",
-            )
-            self.toolsEditor.registerFunction(
-                self.toggleAllVisible, parent=("Overlays",)
-            )
         if img is not None:
             self.setImage(img)
         self.clearOverlays()
 
+    def registerDefaultFunctions(self):
+        self.toolsEditor.registerFunction(self.save)
+        self.toolsEditor.registerFunction(self.toggleImageVisible)
+        self.toolsEditor.registerFunction(self.toggleAllVisible, parent=("Overlays",))
+
     @staticmethod
-    def _createLabelToNameMapping(data=None):
+    def _createLabelNameMap(data=None):
         """
         Helper method to ensure proper dtypes when making a legend series. Helps avoid
         pitfalls with empty data, etc.
@@ -287,7 +287,10 @@ class MaskCompositor(ImageViewer):
         self._updateLegendPos()
 
     def _addRecord(self, rec: pd.Series, update=True):
-        self.recordDf = pd.concat([self.recordDf, rec.to_frame().T], axis=0)
+        # Fix futurewarning from pandas 1.5 regarding bool assignment. It's a false positive,
+        # But the FutureWarnings can crowd the terminal
+        self.recordDf.loc[rec.name] = rec
+        self.recordDf = self.recordDf.astype(dict(isMask="bool"))
         self.curZ += 1
 
         self._addItemCtrls(rec)
@@ -304,7 +307,7 @@ class MaskCompositor(ImageViewer):
         item.setZValue(self.curZ + 1)
         self.viewbox.addItem(item)
 
-        rec = pd.Series(newRecord, name=name)
+        rec = pd.Series(newRecord, name=name, dtype=object)
         self._addRecord(rec, update)
 
     def _getUniqueName(self, baseName: str = None):
@@ -405,11 +408,11 @@ class MaskCompositor(ImageViewer):
             If *True*, the image overlay properties will be refereshed
         """
         if useFullLabelMap:
-            labels = np.union1d(self.inUseLabelValues, self.labelToNameMapping.index)
+            labels = np.union1d(self.inUseLabelValues, self.labelNameMap.index)
         else:
             labels = self.inUseLabelValues
         labels = np.setdiff1d(labels, self.backgroundValues)
-        names = self.labelToNameMapping.reindex(labels)
+        names = self.labelNameMap.reindex(labels)
         needsEntry = names.isna()
         names[needsEntry] = names.index[needsEntry].map(str)
         self.legend.clear()
@@ -450,7 +453,7 @@ class MaskCompositor(ImageViewer):
         if np.isscalar(backgroundValues):
             backgroundValues = [backgroundValues]
         self.backgroundValues = np.array(backgroundValues)
-        self.labelToNameMapping = self._createLabelToNameMapping(labelMap)
+        self.labelNameMap = self._createLabelNameMap(labelMap)
         if updateOverlays:
             self._updateGraphics()
 
@@ -470,14 +473,14 @@ class MaskCompositor(ImageViewer):
             Opacity of the overlay, from 0 (transparent) to 1 (opaque)
         fullLabelMapColors
             If *True*, enough colors will be generated for all labels in
-            ``self.labelToNameMapping``. If *False*, only enough colors will be
+            ``self.labelNameMap``. If *False*, only enough colors will be
             generated for the labels in the added masks.
         """
         maskIdxs = self.recordDf["isMask"].to_numpy(bool)
         cmap = fns.getAnyPgColormap(colormap)
         labelValues = self.scatterSer.index.values
         if fullLabelMapColors:
-            labelValues = np.union1d(labelValues, self.labelToNameMapping.index.values)
+            labelValues = np.union1d(labelValues, self.labelNameMap.index.values)
         if not len(labelValues):
             pass  # return
         if cmap is None:
@@ -486,14 +489,11 @@ class MaskCompositor(ImageViewer):
         colors[:, -1] = opacity * 255
         # If a labelmap has entries, try to scale the colormap to contain all
         # possibilities
-        numEntries = (
-            max(
-                np.max(labelValues, initial=-1),
-                np.max(self.labelToNameMapping.index.values, initial=-1),
-            )
-            + 1
+        numEntries = max(
+            np.max(labelValues, initial=1),
+            np.max(self.labelNameMap.index.to_numpy(), initial=1),
         )
-        lut = np.zeros((numEntries, 4), dtype="uint8")
+        lut = np.zeros((numEntries + 1, 4), dtype="uint8")
         lut[labelValues] = colors
         bgValsInLut = self.backgroundValues[self.backgroundValues < len(lut)]
         lut[bgValsInLut] = 0
@@ -507,7 +507,7 @@ class MaskCompositor(ImageViewer):
             if itemOpacity < 0:
                 itemOpacity = opacity
             maskRec["item"].setOpts(
-                lut=lut, opacity=itemOpacity, levels=[0, numEntries - 1]
+                lut=lut, opacity=itemOpacity, levels=[0, numEntries]
             )
         # Handle non-mask items who still want controlled opacity
         for _, rec in self.recordDf[
@@ -644,6 +644,12 @@ class MaskCompositor(ImageViewer):
             self.legend.setParentItem(self.plotItem)
             self.legend.autoAnchor(oldPos, relative=False)
 
+    def toggleLegendVisible(self):
+        self.legend.setVisible(not self.legend.isVisible())
+
+    def toggleImageVisible(self):
+        self.imageItem.setVisible(not self.imageItem.isVisible())
+
     def __getstate__(self):
         ret = dict(
             image=self.imageItem.image,
