@@ -99,7 +99,7 @@ class PopupLineEditor(QtWidgets.QLineEdit):
         popup.setFocus()
 
     def event(self, ev: QtCore.QEvent):
-        if ev.type() != QtCore.QEvent.Type.KeyPress:
+        if not hasattr(ev, "type") or ev.type() != QtCore.QEvent.Type.KeyPress:
             return super().event(ev)
 
         ev: QtGui.QKeyEvent
