@@ -220,7 +220,7 @@ class FadeNotifyHandler(TimedMessageHandler):
         parent: QtWidgets.QMainWindow = self.getWindow()
         if parent is not None:
             metrics = fadeMsg.fontMetrics()
-            padding = (parent.width() - metrics.width(msg)) // 2
+            padding = (parent.width() - metrics.boundingRect(msg).width()) // 2
             msgX = padding
             # Make sure multiple messages don't overlap
             msgY = int(0.05 * parent.height())
