@@ -2,7 +2,7 @@ import pyqtgraph as pg
 from pyqtgraph.Qt import QtWidgets
 
 import qtextras.widgets.easywidget
-from qtextras import ParameterEditor, widgets
+from qtextras import ParameterEditor
 
 
 def dockgrouping(directory):
@@ -27,9 +27,11 @@ def dockgrouping(directory):
 
     return window
 
+
 def test_dockgrouping(tmp_path):
     assert dockgrouping(tmp_path)
 
+
 if __name__ == "__main__":
     pg.mkQApp()
     win = dockgrouping(".")
